# coding: utf-8

"""
    LEIA RESTful API for AI

    Leia API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Contact: contact@leia.io
    Generated by: https://openapi-generator.tech
"""


from __future__ import annotations
from inspect import getfullargspec
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import List, Optional
from pydantic import BaseModel, StrictStr
from leiaapi.generated.models.job_result import JobResult
from leiaapi.generated.models.result_types import ResultTypes
from leiaapi.generated.models.statuses import Statuses

class Job(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """
    application_id: StrictStr = ...
    callback_url: Optional[StrictStr] = None
    creation_time: datetime = ...
    document_ids: List[StrictStr] = ...
    execute_after_id: Optional[StrictStr] = None
    finished_time: Optional[datetime] = None
    id: StrictStr = ...
    job_type: StrictStr = ...
    model_id: Optional[StrictStr] = None
    parent_job_id: Optional[StrictStr] = None
    reason: Optional[StrictStr] = None
    result: Optional[JobResult] = None
    result_type: Optional[ResultTypes] = None
    starting_time: Optional[datetime] = None
    status: Statuses = ...
    __properties = ["application_id", "callback_url", "creation_time", "document_ids", "execute_after_id", "finished_time", "id", "job_type", "model_id", "parent_job_id", "reason", "result", "result_type", "starting_time", "status"]

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Job:
        """Create an instance of Job from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of result
        if self.result:
            _dict['result'] = self.result.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Job:
        """Create an instance of Job from a dict"""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Job.parse_obj(obj)

        _obj = Job.parse_obj({
            "application_id": obj.get("application_id"),
            "callback_url": obj.get("callback_url"),
            "creation_time": obj.get("creation_time"),
            "document_ids": obj.get("document_ids"),
            "execute_after_id": obj.get("execute_after_id"),
            "finished_time": obj.get("finished_time"),
            "id": obj.get("id"),
            "job_type": obj.get("job_type"),
            "model_id": obj.get("model_id"),
            "parent_job_id": obj.get("parent_job_id"),
            "reason": obj.get("reason"),
            "result": JobResult.from_dict(obj.get("result")) if obj.get("result") is not None else None,
            "result_type": obj.get("result_type"),
            "starting_time": obj.get("starting_time"),
            "status": obj.get("status")
        })
        return _obj




import logging
import sched
import time
from threading import Thread, Lock

logger = logging.getLogger(__name__)


class Scheduler:
    def __call__(self, func, *args, **kwargs):
        logger.info(f'Scheduling method {func.__name__} every {self.interval} seconds')
        self.func = func
        self.run()
        return self

    def __init__(self, interval, *args, **kwargs):
        self.interval = interval
        self.func = None
        self.args = args
        self.kwargs = kwargs
        self.scheduler = sched.scheduler(time.time, time.sleep)
        logger.info(f'Setup Scheduler every {self.interval} seconds')

        self.last_id = None
        self.thread = None

    def periodic(self):
        self.last_id = self.scheduler.enter(self.interval, 1, self.periodic, ())
        self.func(*self.args, **self.kwargs)

    def local_run(self):
        self.periodic()
        self.scheduler.run()

    def run(self):
        self.thread = Thread(target=self.local_run, name=f'schedule-{self.func.__name__}')
        self.thread.daemon = True
        self.thread.start()

    def cancel(self):
        logger.info(f'Cancel Scheduler for {self.func.__name__}')
        self.scheduler.cancel(self.last_id)


def scheduled(interval):
    return Scheduler(interval)



import io
import logging
from pathlib import Path
from typing import Optional, Union, Dict, List, Tuple, Set, Callable

from tqdm.auto import tqdm
from urllib3 import HTTPResponse

from leiaapi.generated.api import ApplicationAdminApi, ModelAdminApi
from leiaapi.generated.api_client import ApiClient
from leiaapi.generated.models import Application, Model
from leiaapi.generated.rest import ApiException

logger = logging.getLogger(__name__)


class FileIncompleteError(BaseException):
    pass


def download_model(token: str, model: Union[Model, Tuple[str, str], Tuple[str, str, Optional[int]]], save_folder: Optional[Union[Path, str]] = None, overwrite: bool = False, use_tqdm: bool = True, client: Optional[ApiClient] = None,
                   fn_filename: Callable[[Union[Model, Tuple[str, str], Tuple[str, str, Optional[int]]]], Union[str, Path]] = lambda model: f"{model.name}.model"):
    if isinstance(model, Model):
        model_id = model.id
        application_id = model.application_id
        total_size = int(model.size)
    elif isinstance(model, tuple):
        if len(model) == 2:
            model_id, application_id = model
            total_size = None
        else:
            model_id, application_id, total_size = model
    else:
        raise ValueError("model parameter not valid")

    file_name = fn_filename(model)
    api = ModelAdminApi(client)

    if save_folder is not None:
        if not isinstance(save_folder, Path):
            save_folder = Path(save_folder)
        if (save_folder / file_name).exists() and (total_size is None or total_size < 0 or total_size == (save_folder / file_name).stat().st_size) and not overwrite:
            logger.info(f'ignore {model_id}, same file already exist on disk')
            return save_folder / file_name

        if not save_folder.is_dir():
            save_folder.mkdir(parents=True, exist_ok=True)

        logger.info(f"Start downloading Model: {model_id} - Application: {application_id} - FileName: {save_folder / file_name} - size: {model_id} ({total_size / 1024 / 1024}Mo)")
    else:
        logger.info(f"Start downloading Model: {model_id} - Application: {application_id} - In memory: {file_name} - size: {model_id} ({total_size / 1024 / 1024}Mo)")
    resp: Optional[HTTPResponse] = None
    while resp is None or resp.status != 200:
        # logger.info(f'Calling {api.admin_get_model_contents.settings["endpoint_path"].format(**model._data_store, model_id=model.id)}')

        resp = api.admin_get_model_contents_with_http_info(token, model.application_id, model.id)

        if resp.status != 200:
            # if exit_on_error:
            # raise Exception(
            #     f'Cannot download {api.admin_get_model_contents.settings["endpoint_path"].format(**model._data_store, model_id=model.id)}, RETURN: {resp.status}')
            pass

    if not use_tqdm:
        logger.info(f"Downloading {model_id} with size {total_size}")

    try:
        if save_folder is None:
            f = io.BytesIO()
            read_stream(resp, f, use_tqdm)
            res = f.getvalue()
        else:
            with (save_folder / file_name).open("wb") as f:
                read_stream(resp, f, use_tqdm)

            res = save_folder / file_name
    except FileIncompleteError as e:
        if save_folder is not None:
            logger.info(f"Deleting incomplete file {(save_folder / file_name)}")
            (save_folder / file_name).unlink()
        raise FileIncompleteError(f"Cannot download {model_id}") from e

    logger.info(f"Download of {model_id} successful")

    return res


def read_stream(resp: HTTPResponse, stream, use_tqdm):
    total_size = int(resp.headers.get('content-length', 0))

    with tqdm(total=total_size, unit='iB', unit_scale=True, disable=not use_tqdm, unit_divisor=1024) as t:
        remaining = total_size

        for block in resp.stream(32768):
            remaining -= len(block)
            t.update(len(block))
            stream.write(block)

    if remaining > 0:
        raise FileIncompleteError(f"Missing {remaining} Bytes")


def get_all_models(token: str, client: Optional[ApiClient] = None) -> List[Model]:
    total = -1
    models = []
    model_api = ModelAdminApi(client)
    status_code = None
    while len(models) < total or total < 0:
        try:
            resp, status_code, headers = model_api.admin_get_models_with_http_info(token, offset=len(models))
        except ApiException as e:
            if e.status == 404:
                logger.warning("API return a 404, maybe because the URl is wrong or because there is no model available")
            return []

        if status_code != 200:
            raise Exception(f'GET {resp.url} FAILED: {status_code}')

        firstlast, total = headers["Content-Range"].split("/")
        total = int(total)
        logger.info(f"Model pagination: {len(models)}/{total}")

        models.extend(resp)
    logger.info(f"Models found: {len(models)}")
    if len(models) != total:
        raise Exception(f'Receive more or less data than expected : {len(models)} instead of {total}')
    return models

def get_all_application(token: str, client: Optional[ApiClient] = None) -> List[Model]:
    total = -1
    models = []
    application_api = ApplicationAdminApi(client)
    status_code = None
    while len(models) < total or total < 0:
        try:
            resp, status_code, headers = application_api.admin_get_applications_with_http_info(token, offset=len(models))
        except ApiException as e:
            if e.status == 404:
                logger.warning("API return a 404, maybe because the URl is wrong or because there is no application available")
            return []

        if status_code != 200:
            raise Exception(f'GET {resp.url} FAILED: {status_code}')

        firstlast, total = headers["Content-Range"].split("/")
        total = int(total)
        logger.info(f"Model pagination: {len(models)}/{total}")

        models.extend(resp)
    logger.info(f"Models found: {len(models)}")
    if len(models) != total:
        raise Exception(f'Receive more or less data than expected : {len(models)} instead of {total}')
    return models

def download_models(token, save_folder: Optional[Union[Path, str]] = None, whitelist: Set[str] = (), blacklist: Set[str] = (), overwrite: bool = False,
                    use_tqdm: bool = True, keep_model: bool = True, stop_if_error: bool = False, client: Optional[ApiClient] = None,
                    fn_subfolder: Callable[[Application, Model], Union[str, Path]] = lambda application, model: application.application_name,
                    fn_filename: Callable[[Union[Model, Tuple[str, str], Tuple[str, str, Optional[int]]]], Union[str, Path]] = lambda model: f"{model.name}.model"):
    # if len(whitelist) > 0 and len(blacklist) > 0:
    #     raise ValueError("whitelist and blacklist cannot be set at the same time")

    applications: Dict[str, Application] = {app.id: app for app in get_all_application(token, client)}
    print(applications.keys())
    models: List[Model] = get_all_models(token=token, client=client)

    if save_folder is not None:
        if not isinstance(save_folder, Path):
            save_folder = Path(save_folder)

    if whitelist:
        models = [model for model in models if len({model.name, model.short_name, model.id, model.md5sum}.intersection(whitelist)) > 0]
    if blacklist:
        models = [model for model in models if len({model.name, model.short_name, model.id, model.md5sum}.intersection(blacklist)) == 0]

    for model in models:
        try:
            if save_folder is not None:
                folder = save_folder / fn_subfolder(applications[model.application_id], model)
            else:
                folder = None
            file = download_model(token=token, model=model, save_folder=folder, overwrite=overwrite, use_tqdm=use_tqdm, client=client, fn_filename=fn_filename)
            if file is None:
                continue
            try:
                yield model, file
            finally:
                if not keep_model:
                    file.unlink()
        except FileIncompleteError as e:
            logger.exception(f"Cannot download model {model.name} - {model.id} - {model.short_name}")
            if stop_if_error:
                raise
        except Exception as e:
            logger.exception("")
            if stop_if_error:
                raise



# coding: utf-8

"""
    LEIA RESTful API for AI

    Leia API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Contact: contact@leia.io
    Generated by: https://openapi-generator.tech
"""


from __future__ import annotations
from inspect import getfullargspec
import json
import pprint
import re  # noqa: F401

from typing import Any, List, Optional
from pydantic import BaseModel, Field, StrictStr, ValidationError, validator
from leiaapi.generated.models.classification import Classification
from leiaapi.generated.models.document import Document
from typing import Any, List
from pydantic import StrictStr, Field

JOBRESULT_ONE_OF_SCHEMAS = ["Classification", "Document", "List[Document]", "object"]

class JobResult(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """
    # data type: Document
    oneof_schema_1_validator: Optional[Document] = None
    # data type: List[Document]
    oneof_schema_2_validator: Optional[List[Document]] = None
    # data type: Classification
    oneof_schema_3_validator: Optional[Classification] = None
    # data type: object
    oneof_schema_4_validator: Optional[Any] = None
    actual_instance: Any
    one_of_schemas: List[str] = Field(JOBRESULT_ONE_OF_SCHEMAS, const=True)

    class Config:
        validate_assignment = True

    @validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        instance = cls()
        error_messages = []
        match = 0
        # validate data type: Document
        if type(v) is not Document:
            error_messages.append(f"Error! Input type `{type(v)}` is not `Document`")
        else:
            match += 1

        # validate data type: List[Document]
        try:
            instance.oneof_schema_2_validator = v
            match += 1
        except ValidationError as e:
            error_messages.append(str(e))
        # validate data type: Classification
        if type(v) is not Classification:
            error_messages.append(f"Error! Input type `{type(v)}` is not `Classification`")
        else:
            match += 1

        if match == 0:
            # validate data type: object
            try:
                instance.oneof_schema_4_validator = v
                match += 1
            except ValidationError as e:
                error_messages.append(str(e))
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into JobResult with oneOf schemas: Classification, Document, List[Document], object. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into JobResult with oneOf schemas: Classification, Document, List[Document], object. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: dict) -> JobResult:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> JobResult:
        """Returns the object represented by the json string"""
        instance = cls()
        error_messages = []
        match = 0

        # deserialize data into Document
        try:
            instance.actual_instance = Document.from_json(json_str)
            match += 1
        except ValidationError as e:
            error_messages.append(str(e))
        # deserialize data into List[Document]
        try:
            # validation
            instance.oneof_schema_2_validator = json.loads(json_str)
            # assign value to actual_instance
            instance.actual_instance = instance.oneof_schema_2_validator
            match += 1
        except ValidationError as e:
            error_messages.append(str(e))
        # deserialize data into Classification
        try:
            instance.actual_instance = Classification.from_json(json_str)
            match += 1
        except ValidationError as e:
            error_messages.append(str(e))

        if match == 0:
            # deserialize data into object
            try:
                # validation
                instance.oneof_schema_4_validator = json.loads(json_str)
                # assign value to actual_instance
                instance.actual_instance = instance.oneof_schema_4_validator
                match += 1
            except ValidationError as e:
                error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into JobResult with oneOf schemas: Classification, Document, List[Document], object. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into JobResult with oneOf schemas: Classification, Document, List[Document], object. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is not None:
            return self.actual_instance.to_json()
        else:
            return "null"

    def to_dict(self) -> dict:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is not None:
            return self.actual_instance.to_dict()
        else:
            return dict()

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.dict())




# coding: utf-8

"""
    LEIA RESTful API for AI

    Leia API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Contact: contact@leia.io
    Generated by: https://openapi-generator.tech
"""


from __future__ import annotations
from inspect import getfullargspec
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Any, Dict, List, Optional
from pydantic import BaseModel, StrictStr
from leiaapi.generated.models.annotation_types import AnnotationTypes

class Annotation(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """
    annotation_type: AnnotationTypes = ...
    application_id: StrictStr = ...
    creation_time: datetime = ...
    document_id: StrictStr = ...
    id: StrictStr = ...
    name: Optional[StrictStr] = None
    prediction: Optional[Dict[str, Any]] = None
    tags: Optional[List[StrictStr]] = None
    __properties = ["annotation_type", "application_id", "creation_time", "document_id", "id", "name", "prediction", "tags"]

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Annotation:
        """Create an instance of Annotation from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Annotation:
        """Create an instance of Annotation from a dict"""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Annotation.parse_obj(obj)

        _obj = Annotation.parse_obj({
            "annotation_type": obj.get("annotation_type"),
            "application_id": obj.get("application_id"),
            "creation_time": obj.get("creation_time"),
            "document_id": obj.get("document_id"),
            "id": obj.get("id"),
            "name": obj.get("name"),
            "prediction": obj.get("prediction"),
            "tags": obj.get("tags")
        })
        return _obj




# coding: utf-8

"""
    LEIA RESTful API for AI

    Leia API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Contact: contact@leia.io
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

from pydantic import validate_arguments, ValidationError
from typing_extensions import Annotated

from datetime import datetime

from pydantic import Field, StrictBool, StrictInt, StrictStr

from typing import Dict, List, Optional

from leiaapi.generated.models.document import Document
from leiaapi.generated.models.job import Job
from leiaapi.generated.models.transform_body import TransformBody
from leiaapi.generated.models.transform_types import TransformTypes

from leiaapi.generated.api_client import ApiClient
from leiaapi.generated.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class DocumentAdminApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def admin_create_document(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[StrictStr, Field(..., description="The application that will own the model")], filename : Annotated[StrictStr, Field(..., description="The name of the file (if present, extension will be separated from filename in metadata of the document)")], body : StrictStr, ttl : Annotated[Optional[StrictInt], Field(description="The TTL (in seconds, not less than 60) for the document (if present, the document and any sub documents, annotations, or jobs linked to it will be deleted after the TTL is expired)")] = None, tags : Annotated[Optional[List[StrictStr]], Field(description="The tags of the document")] = None, **kwargs) -> Document:  # noqa: E501
        """Uploads a document to the Leia API (admin only)  # noqa: E501

        Uploads a document to Leia API for future use. This method is only accessible to admins   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_create_document(token, application_id, filename, body, ttl, tags, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: The application that will own the model (required)
        :type application_id: str
        :param filename: The name of the file (if present, extension will be separated from filename in metadata of the document) (required)
        :type filename: str
        :param body: (required)
        :type body: str
        :param ttl: The TTL (in seconds, not less than 60) for the document (if present, the document and any sub documents, annotations, or jobs linked to it will be deleted after the TTL is expired)
        :type ttl: int
        :param tags: The tags of the document
        :type tags: List[str]
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Document
        """
        kwargs['_return_http_data_only'] = True
        return self.admin_create_document_with_http_info(token, application_id, filename, body, ttl, tags, **kwargs)  # noqa: E501

    @validate_arguments
    def admin_create_document_with_http_info(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[StrictStr, Field(..., description="The application that will own the model")], filename : Annotated[StrictStr, Field(..., description="The name of the file (if present, extension will be separated from filename in metadata of the document)")], body : StrictStr, ttl : Annotated[Optional[StrictInt], Field(description="The TTL (in seconds, not less than 60) for the document (if present, the document and any sub documents, annotations, or jobs linked to it will be deleted after the TTL is expired)")] = None, tags : Annotated[Optional[List[StrictStr]], Field(description="The tags of the document")] = None, **kwargs):  # noqa: E501
        """Uploads a document to the Leia API (admin only)  # noqa: E501

        Uploads a document to Leia API for future use. This method is only accessible to admins   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_create_document_with_http_info(token, application_id, filename, body, ttl, tags, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: The application that will own the model (required)
        :type application_id: str
        :param filename: The name of the file (if present, extension will be separated from filename in metadata of the document) (required)
        :type filename: str
        :param body: (required)
        :type body: str
        :param ttl: The TTL (in seconds, not less than 60) for the document (if present, the document and any sub documents, annotations, or jobs linked to it will be deleted after the TTL is expired)
        :type ttl: int
        :param tags: The tags of the document
        :type tags: List[str]
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Document, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'token',
            'application_id',
            'filename',
            'body',
            'ttl',
            'tags'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method admin_create_document" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['application_id']:
            _path_params['application_id'] = _params['application_id']

        # process the query parameters
        _query_params = []
        if _params.get('filename') is not None:  # noqa: E501
            _query_params.append(('filename', _params['filename']))
        if _params.get('ttl') is not None:  # noqa: E501
            _query_params.append(('ttl', _params['ttl']))
        if _params.get('tags') is not None:  # noqa: E501
            _query_params.append(('tags', _params['tags']))
            _collection_formats['tags'] = 'multi'

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['token']:
            _header_params['token'] = _params['token']

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None
        if _params['body']:
            _body_params = _params['body']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/octet-stream']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "Document",
            '401': None,
        }

        return self.api_client.call_api(
            '/admin/{application_id}/document', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def admin_delete_document(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[StrictStr, Field(..., description="The application to which the document to delete belongs")], document_id : Annotated[StrictStr, Field(..., description="The id of the document to delete")], **kwargs) -> None:  # noqa: E501
        """Deletes a document from Leia API (admin only)  # noqa: E501

        Deletes a document from Leia API. This method is only accessible to admins   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_delete_document(token, application_id, document_id, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: The application to which the document to delete belongs (required)
        :type application_id: str
        :param document_id: The id of the document to delete (required)
        :type document_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs['_return_http_data_only'] = True
        return self.admin_delete_document_with_http_info(token, application_id, document_id, **kwargs)  # noqa: E501

    @validate_arguments
    def admin_delete_document_with_http_info(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[StrictStr, Field(..., description="The application to which the document to delete belongs")], document_id : Annotated[StrictStr, Field(..., description="The id of the document to delete")], **kwargs):  # noqa: E501
        """Deletes a document from Leia API (admin only)  # noqa: E501

        Deletes a document from Leia API. This method is only accessible to admins   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_delete_document_with_http_info(token, application_id, document_id, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: The application to which the document to delete belongs (required)
        :type application_id: str
        :param document_id: The id of the document to delete (required)
        :type document_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = [
            'token',
            'application_id',
            'document_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method admin_delete_document" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['application_id']:
            _path_params['application_id'] = _params['application_id']
        if _params['document_id']:
            _path_params['document_id'] = _params['document_id']

        # process the query parameters
        _query_params = []

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['token']:
            _header_params['token'] = _params['token']

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            '/admin/{application_id}/document/{document_id}', 'DELETE',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def admin_edit_document(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[StrictStr, Field(..., description="The application to which the document to update belongs")], document_id : Annotated[StrictStr, Field(..., description="The id of the document to update")], filename : Annotated[Optional[StrictStr], Field(description="The new file name of the document")] = None, rotation_angle : Annotated[Optional[StrictInt], Field(description="The new rotation angle of the document")] = None, ttl : Annotated[Optional[StrictInt], Field(description="The TTL (in seconds, not less than 60) for the document (if present, the document and any sub documents, annotations, or jobs linked to it will be deleted after the TTL is expired)")] = None, **kwargs) -> Document:  # noqa: E501
        """Updates a document in Leia API (admin only)  # noqa: E501

        Updates metadata for a document. This method is only accessible to admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_edit_document(token, application_id, document_id, filename, rotation_angle, ttl, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: The application to which the document to update belongs (required)
        :type application_id: str
        :param document_id: The id of the document to update (required)
        :type document_id: str
        :param filename: The new file name of the document
        :type filename: str
        :param rotation_angle: The new rotation angle of the document
        :type rotation_angle: int
        :param ttl: The TTL (in seconds, not less than 60) for the document (if present, the document and any sub documents, annotations, or jobs linked to it will be deleted after the TTL is expired)
        :type ttl: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Document
        """
        kwargs['_return_http_data_only'] = True
        return self.admin_edit_document_with_http_info(token, application_id, document_id, filename, rotation_angle, ttl, **kwargs)  # noqa: E501

    @validate_arguments
    def admin_edit_document_with_http_info(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[StrictStr, Field(..., description="The application to which the document to update belongs")], document_id : Annotated[StrictStr, Field(..., description="The id of the document to update")], filename : Annotated[Optional[StrictStr], Field(description="The new file name of the document")] = None, rotation_angle : Annotated[Optional[StrictInt], Field(description="The new rotation angle of the document")] = None, ttl : Annotated[Optional[StrictInt], Field(description="The TTL (in seconds, not less than 60) for the document (if present, the document and any sub documents, annotations, or jobs linked to it will be deleted after the TTL is expired)")] = None, **kwargs):  # noqa: E501
        """Updates a document in Leia API (admin only)  # noqa: E501

        Updates metadata for a document. This method is only accessible to admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_edit_document_with_http_info(token, application_id, document_id, filename, rotation_angle, ttl, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: The application to which the document to update belongs (required)
        :type application_id: str
        :param document_id: The id of the document to update (required)
        :type document_id: str
        :param filename: The new file name of the document
        :type filename: str
        :param rotation_angle: The new rotation angle of the document
        :type rotation_angle: int
        :param ttl: The TTL (in seconds, not less than 60) for the document (if present, the document and any sub documents, annotations, or jobs linked to it will be deleted after the TTL is expired)
        :type ttl: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Document, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'token',
            'application_id',
            'document_id',
            'filename',
            'rotation_angle',
            'ttl'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method admin_edit_document" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['application_id']:
            _path_params['application_id'] = _params['application_id']
        if _params['document_id']:
            _path_params['document_id'] = _params['document_id']

        # process the query parameters
        _query_params = []
        if _params.get('filename') is not None:  # noqa: E501
            _query_params.append(('filename', _params['filename']))
        if _params.get('rotation_angle') is not None:  # noqa: E501
            _query_params.append(('rotation_angle', _params['rotation_angle']))
        if _params.get('ttl') is not None:  # noqa: E501
            _query_params.append(('ttl', _params['ttl']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['token']:
            _header_params['token'] = _params['token']

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "Document",
            '401': None,
            '404': None,
        }

        return self.api_client.call_api(
            '/admin/{application_id}/document/{document_id}', 'PATCH',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def admin_get_document(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[StrictStr, Field(..., description="The application to which the document to retrieve belongs")], document_id : Annotated[StrictStr, Field(..., description="The id of the document to retrieve")], **kwargs) -> Document:  # noqa: E501
        """Retrieves a document from Leia API (admin only)  # noqa: E501

        Retrieves a document from Leia API as metadata. This method is only accessible to admins   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_get_document(token, application_id, document_id, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: The application to which the document to retrieve belongs (required)
        :type application_id: str
        :param document_id: The id of the document to retrieve (required)
        :type document_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Document
        """
        kwargs['_return_http_data_only'] = True
        return self.admin_get_document_with_http_info(token, application_id, document_id, **kwargs)  # noqa: E501

    @validate_arguments
    def admin_get_document_with_http_info(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[StrictStr, Field(..., description="The application to which the document to retrieve belongs")], document_id : Annotated[StrictStr, Field(..., description="The id of the document to retrieve")], **kwargs):  # noqa: E501
        """Retrieves a document from Leia API (admin only)  # noqa: E501

        Retrieves a document from Leia API as metadata. This method is only accessible to admins   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_get_document_with_http_info(token, application_id, document_id, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: The application to which the document to retrieve belongs (required)
        :type application_id: str
        :param document_id: The id of the document to retrieve (required)
        :type document_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Document, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'token',
            'application_id',
            'document_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method admin_get_document" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['application_id']:
            _path_params['application_id'] = _params['application_id']
        if _params['document_id']:
            _path_params['document_id'] = _params['document_id']

        # process the query parameters
        _query_params = []

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['token']:
            _header_params['token'] = _params['token']

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "Document",
            '401': None,
            '404': None,
        }

        return self.api_client.call_api(
            '/admin/{application_id}/document/{document_id}', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def admin_get_document_contents(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[StrictStr, Field(..., description="The application to which the document to retrieve belongs")], document_id : Annotated[StrictStr, Field(..., description="The id of the document to retrieve")], max_size : Annotated[Optional[StrictInt], Field(description="Restrict the size of the image to get (only applicable for documents of type image). The largest dimension of the image will be capped to this dimension")] = None, jpeg_compression : Annotated[Optional[StrictInt], Field(description="JPEG compression rate, in percent (only applicable for documents of type image)")] = None, **kwargs) -> str:  # noqa: E501
        """Retrieves a document from Leia API (admin only)  # noqa: E501

        Retrieves the binary contents of a document from Leia API. This method is only accessible to admins   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_get_document_contents(token, application_id, document_id, max_size, jpeg_compression, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: The application to which the document to retrieve belongs (required)
        :type application_id: str
        :param document_id: The id of the document to retrieve (required)
        :type document_id: str
        :param max_size: Restrict the size of the image to get (only applicable for documents of type image). The largest dimension of the image will be capped to this dimension
        :type max_size: int
        :param jpeg_compression: JPEG compression rate, in percent (only applicable for documents of type image)
        :type jpeg_compression: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: str
        """
        kwargs['_return_http_data_only'] = True
        return self.admin_get_document_contents_with_http_info(token, application_id, document_id, max_size, jpeg_compression, **kwargs)  # noqa: E501

    @validate_arguments
    def admin_get_document_contents_with_http_info(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[StrictStr, Field(..., description="The application to which the document to retrieve belongs")], document_id : Annotated[StrictStr, Field(..., description="The id of the document to retrieve")], max_size : Annotated[Optional[StrictInt], Field(description="Restrict the size of the image to get (only applicable for documents of type image). The largest dimension of the image will be capped to this dimension")] = None, jpeg_compression : Annotated[Optional[StrictInt], Field(description="JPEG compression rate, in percent (only applicable for documents of type image)")] = None, **kwargs):  # noqa: E501
        """Retrieves a document from Leia API (admin only)  # noqa: E501

        Retrieves the binary contents of a document from Leia API. This method is only accessible to admins   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_get_document_contents_with_http_info(token, application_id, document_id, max_size, jpeg_compression, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: The application to which the document to retrieve belongs (required)
        :type application_id: str
        :param document_id: The id of the document to retrieve (required)
        :type document_id: str
        :param max_size: Restrict the size of the image to get (only applicable for documents of type image). The largest dimension of the image will be capped to this dimension
        :type max_size: int
        :param jpeg_compression: JPEG compression rate, in percent (only applicable for documents of type image)
        :type jpeg_compression: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(str, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'token',
            'application_id',
            'document_id',
            'max_size',
            'jpeg_compression'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method admin_get_document_contents" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['application_id']:
            _path_params['application_id'] = _params['application_id']
        if _params['document_id']:
            _path_params['document_id'] = _params['document_id']

        # process the query parameters
        _query_params = []
        if _params.get('max_size') is not None:  # noqa: E501
            _query_params.append(('max_size', _params['max_size']))
        if _params.get('jpeg_compression') is not None:  # noqa: E501
            _query_params.append(('jpeg_compression', _params['jpeg_compression']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['token']:
            _header_params['token'] = _params['token']

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/octet-stream'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "str",
            '401': None,
            '404': None,
        }

        return self.api_client.call_api(
            '/admin/{application_id}/document/{document_id}/file_contents', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def admin_get_documents(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], document_id : Annotated[Optional[StrictStr], Field(description="The id of the document")] = None, application_id : Annotated[Optional[StrictStr], Field(description="The application that owns the documents")] = None, filename : Annotated[Optional[StrictStr], Field(description="The file name of the documents to retrieve")] = None, extension : Annotated[Optional[StrictStr], Field(description="The extension of the documents to retrieve")] = None, mime_type : Annotated[Optional[StrictStr], Field(description="Filters by MIME type")] = None, original_id : Annotated[Optional[StrictStr], Field(description="Filters by original id")] = None, tags : Annotated[Optional[List[StrictStr]], Field(description="If specified, filters the documents by tag")] = None, created_after : Annotated[Optional[datetime], Field(description="If specified, keeps only documents created after given UTC timestamp (ISO 8601 format : yyyy-MM-ddThh:mm:ss)")] = None, created_before : Annotated[Optional[datetime], Field(description="If specified, keeps only documents created before given UTC timestamp (ISO 8601 format : yyyy-MM-ddThh:mm:ss)")] = None, tag_result : Annotated[Optional[StrictStr], Field(description="Atomically adds a tag to all retrieved values if specified. The added tag will not be returned in the result")] = None, sort : Annotated[Optional[StrictStr], Field(description="If specified, sorts the documents by a list of existing parameters separated by commas. Can be 'application_id', 'filename', 'extension', 'mime_type', 'original_id', 'page', 'creation_time'. Sorts in ascending order by default. If a parameter is preceded by '-', it is sorted in descending order.")] = None, offset : Annotated[Optional[StrictInt], Field(description="Number of the first document to send (pagination)")] = None, limit : Annotated[Optional[StrictInt], Field(description="Maximum number of documents to send (pagination)")] = None, **kwargs) -> List[Document]:  # noqa: E501
        """Retrieves documents from Leia API (admin only) (paginated)  # noqa: E501

        Retrieves documents which matches the query from Leia API as JSON metadata. This method is only accessible to admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_get_documents(token, document_id, application_id, filename, extension, mime_type, original_id, tags, created_after, created_before, tag_result, sort, offset, limit, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param document_id: The id of the document
        :type document_id: str
        :param application_id: The application that owns the documents
        :type application_id: str
        :param filename: The file name of the documents to retrieve
        :type filename: str
        :param extension: The extension of the documents to retrieve
        :type extension: str
        :param mime_type: Filters by MIME type
        :type mime_type: str
        :param original_id: Filters by original id
        :type original_id: str
        :param tags: If specified, filters the documents by tag
        :type tags: List[str]
        :param created_after: If specified, keeps only documents created after given UTC timestamp (ISO 8601 format : yyyy-MM-ddThh:mm:ss)
        :type created_after: datetime
        :param created_before: If specified, keeps only documents created before given UTC timestamp (ISO 8601 format : yyyy-MM-ddThh:mm:ss)
        :type created_before: datetime
        :param tag_result: Atomically adds a tag to all retrieved values if specified. The added tag will not be returned in the result
        :type tag_result: str
        :param sort: If specified, sorts the documents by a list of existing parameters separated by commas. Can be 'application_id', 'filename', 'extension', 'mime_type', 'original_id', 'page', 'creation_time'. Sorts in ascending order by default. If a parameter is preceded by '-', it is sorted in descending order.
        :type sort: str
        :param offset: Number of the first document to send (pagination)
        :type offset: int
        :param limit: Maximum number of documents to send (pagination)
        :type limit: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: List[Document]
        """
        kwargs['_return_http_data_only'] = True
        return self.admin_get_documents_with_http_info(token, document_id, application_id, filename, extension, mime_type, original_id, tags, created_after, created_before, tag_result, sort, offset, limit, **kwargs)  # noqa: E501

    @validate_arguments
    def admin_get_documents_with_http_info(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], document_id : Annotated[Optional[StrictStr], Field(description="The id of the document")] = None, application_id : Annotated[Optional[StrictStr], Field(description="The application that owns the documents")] = None, filename : Annotated[Optional[StrictStr], Field(description="The file name of the documents to retrieve")] = None, extension : Annotated[Optional[StrictStr], Field(description="The extension of the documents to retrieve")] = None, mime_type : Annotated[Optional[StrictStr], Field(description="Filters by MIME type")] = None, original_id : Annotated[Optional[StrictStr], Field(description="Filters by original id")] = None, tags : Annotated[Optional[List[StrictStr]], Field(description="If specified, filters the documents by tag")] = None, created_after : Annotated[Optional[datetime], Field(description="If specified, keeps only documents created after given UTC timestamp (ISO 8601 format : yyyy-MM-ddThh:mm:ss)")] = None, created_before : Annotated[Optional[datetime], Field(description="If specified, keeps only documents created before given UTC timestamp (ISO 8601 format : yyyy-MM-ddThh:mm:ss)")] = None, tag_result : Annotated[Optional[StrictStr], Field(description="Atomically adds a tag to all retrieved values if specified. The added tag will not be returned in the result")] = None, sort : Annotated[Optional[StrictStr], Field(description="If specified, sorts the documents by a list of existing parameters separated by commas. Can be 'application_id', 'filename', 'extension', 'mime_type', 'original_id', 'page', 'creation_time'. Sorts in ascending order by default. If a parameter is preceded by '-', it is sorted in descending order.")] = None, offset : Annotated[Optional[StrictInt], Field(description="Number of the first document to send (pagination)")] = None, limit : Annotated[Optional[StrictInt], Field(description="Maximum number of documents to send (pagination)")] = None, **kwargs):  # noqa: E501
        """Retrieves documents from Leia API (admin only) (paginated)  # noqa: E501

        Retrieves documents which matches the query from Leia API as JSON metadata. This method is only accessible to admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_get_documents_with_http_info(token, document_id, application_id, filename, extension, mime_type, original_id, tags, created_after, created_before, tag_result, sort, offset, limit, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param document_id: The id of the document
        :type document_id: str
        :param application_id: The application that owns the documents
        :type application_id: str
        :param filename: The file name of the documents to retrieve
        :type filename: str
        :param extension: The extension of the documents to retrieve
        :type extension: str
        :param mime_type: Filters by MIME type
        :type mime_type: str
        :param original_id: Filters by original id
        :type original_id: str
        :param tags: If specified, filters the documents by tag
        :type tags: List[str]
        :param created_after: If specified, keeps only documents created after given UTC timestamp (ISO 8601 format : yyyy-MM-ddThh:mm:ss)
        :type created_after: datetime
        :param created_before: If specified, keeps only documents created before given UTC timestamp (ISO 8601 format : yyyy-MM-ddThh:mm:ss)
        :type created_before: datetime
        :param tag_result: Atomically adds a tag to all retrieved values if specified. The added tag will not be returned in the result
        :type tag_result: str
        :param sort: If specified, sorts the documents by a list of existing parameters separated by commas. Can be 'application_id', 'filename', 'extension', 'mime_type', 'original_id', 'page', 'creation_time'. Sorts in ascending order by default. If a parameter is preceded by '-', it is sorted in descending order.
        :type sort: str
        :param offset: Number of the first document to send (pagination)
        :type offset: int
        :param limit: Maximum number of documents to send (pagination)
        :type limit: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(List[Document], status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'token',
            'document_id',
            'application_id',
            'filename',
            'extension',
            'mime_type',
            'original_id',
            'tags',
            'created_after',
            'created_before',
            'tag_result',
            'sort',
            'offset',
            'limit'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method admin_get_documents" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('document_id') is not None:  # noqa: E501
            _query_params.append(('document_id', _params['document_id']))
        if _params.get('application_id') is not None:  # noqa: E501
            _query_params.append(('application_id', _params['application_id']))
        if _params.get('filename') is not None:  # noqa: E501
            _query_params.append(('filename', _params['filename']))
        if _params.get('extension') is not None:  # noqa: E501
            _query_params.append(('extension', _params['extension']))
        if _params.get('mime_type') is not None:  # noqa: E501
            _query_params.append(('mime_type', _params['mime_type']))
        if _params.get('original_id') is not None:  # noqa: E501
            _query_params.append(('original_id', _params['original_id']))
        if _params.get('tags') is not None:  # noqa: E501
            _query_params.append(('tags', _params['tags']))
            _collection_formats['tags'] = 'multi'
        if _params.get('created_after') is not None:  # noqa: E501
            _query_params.append(('created_after', _params['created_after']))
        if _params.get('created_before') is not None:  # noqa: E501
            _query_params.append(('created_before', _params['created_before']))
        if _params.get('tag_result') is not None:  # noqa: E501
            _query_params.append(('tag_result', _params['tag_result']))
        if _params.get('sort') is not None:  # noqa: E501
            _query_params.append(('sort', _params['sort']))
        if _params.get('offset') is not None:  # noqa: E501
            _query_params.append(('offset', _params['offset']))
        if _params.get('limit') is not None:  # noqa: E501
            _query_params.append(('limit', _params['limit']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['token']:
            _header_params['token'] = _params['token']

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "List[Document]",
            '401': None,
            '403': None,
            '404': None,
        }

        return self.api_client.call_api(
            '/admin/document', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def admin_get_documents_tags(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[Optional[StrictStr], Field(description="Restrict to tags belonging to this application")] = None, **kwargs) -> List[str]:  # noqa: E501
        """Retrieves documents' tags from Leia API (admin only)  # noqa: E501

        Retrieves tags from documents. This method is only accessible to admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_get_documents_tags(token, application_id, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: Restrict to tags belonging to this application
        :type application_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: List[str]
        """
        kwargs['_return_http_data_only'] = True
        return self.admin_get_documents_tags_with_http_info(token, application_id, **kwargs)  # noqa: E501

    @validate_arguments
    def admin_get_documents_tags_with_http_info(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[Optional[StrictStr], Field(description="Restrict to tags belonging to this application")] = None, **kwargs):  # noqa: E501
        """Retrieves documents' tags from Leia API (admin only)  # noqa: E501

        Retrieves tags from documents. This method is only accessible to admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_get_documents_tags_with_http_info(token, application_id, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: Restrict to tags belonging to this application
        :type application_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(List[str], status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'token',
            'application_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method admin_get_documents_tags" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('application_id') is not None:  # noqa: E501
            _query_params.append(('application_id', _params['application_id']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['token']:
            _header_params['token'] = _params['token']

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "List[str]",
            '401': None,
            '404': None,
        }

        return self.api_client.call_api(
            '/admin/document/tag', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def admin_get_documents_zip(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], document_id : Annotated[Optional[StrictStr], Field(description="The id of the document")] = None, application_id : Annotated[Optional[StrictStr], Field(description="The application that owns the documents")] = None, filename : Annotated[Optional[StrictStr], Field(description="The file name of the documents to retrieve")] = None, extension : Annotated[Optional[StrictStr], Field(description="The extension of the documents to retrieve")] = None, mime_type : Annotated[Optional[StrictStr], Field(description="Filters by MIME type")] = None, original_id : Annotated[Optional[StrictStr], Field(description="Filters by original id")] = None, tags : Annotated[Optional[List[StrictStr]], Field(description="If specified, filters the documents by tag")] = None, created_after : Annotated[Optional[datetime], Field(description="If specified, keeps only documents created after given UTC timestamp (ISO 8601 format : yyyy-MM-ddThh:mm:ss)")] = None, created_before : Annotated[Optional[datetime], Field(description="If specified, keeps only documents created before given UTC timestamp (ISO 8601 format : yyyy-MM-ddThh:mm:ss)")] = None, **kwargs) -> str:  # noqa: E501
        """Retrieves documents from Leia API (admin only) (paginated)  # noqa: E501

        Retrieves documents which matches the query from Leia API in a zip file. This method is only accessible to admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_get_documents_zip(token, document_id, application_id, filename, extension, mime_type, original_id, tags, created_after, created_before, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param document_id: The id of the document
        :type document_id: str
        :param application_id: The application that owns the documents
        :type application_id: str
        :param filename: The file name of the documents to retrieve
        :type filename: str
        :param extension: The extension of the documents to retrieve
        :type extension: str
        :param mime_type: Filters by MIME type
        :type mime_type: str
        :param original_id: Filters by original id
        :type original_id: str
        :param tags: If specified, filters the documents by tag
        :type tags: List[str]
        :param created_after: If specified, keeps only documents created after given UTC timestamp (ISO 8601 format : yyyy-MM-ddThh:mm:ss)
        :type created_after: datetime
        :param created_before: If specified, keeps only documents created before given UTC timestamp (ISO 8601 format : yyyy-MM-ddThh:mm:ss)
        :type created_before: datetime
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: str
        """
        kwargs['_return_http_data_only'] = True
        return self.admin_get_documents_zip_with_http_info(token, document_id, application_id, filename, extension, mime_type, original_id, tags, created_after, created_before, **kwargs)  # noqa: E501

    @validate_arguments
    def admin_get_documents_zip_with_http_info(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], document_id : Annotated[Optional[StrictStr], Field(description="The id of the document")] = None, application_id : Annotated[Optional[StrictStr], Field(description="The application that owns the documents")] = None, filename : Annotated[Optional[StrictStr], Field(description="The file name of the documents to retrieve")] = None, extension : Annotated[Optional[StrictStr], Field(description="The extension of the documents to retrieve")] = None, mime_type : Annotated[Optional[StrictStr], Field(description="Filters by MIME type")] = None, original_id : Annotated[Optional[StrictStr], Field(description="Filters by original id")] = None, tags : Annotated[Optional[List[StrictStr]], Field(description="If specified, filters the documents by tag")] = None, created_after : Annotated[Optional[datetime], Field(description="If specified, keeps only documents created after given UTC timestamp (ISO 8601 format : yyyy-MM-ddThh:mm:ss)")] = None, created_before : Annotated[Optional[datetime], Field(description="If specified, keeps only documents created before given UTC timestamp (ISO 8601 format : yyyy-MM-ddThh:mm:ss)")] = None, **kwargs):  # noqa: E501
        """Retrieves documents from Leia API (admin only) (paginated)  # noqa: E501

        Retrieves documents which matches the query from Leia API in a zip file. This method is only accessible to admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_get_documents_zip_with_http_info(token, document_id, application_id, filename, extension, mime_type, original_id, tags, created_after, created_before, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param document_id: The id of the document
        :type document_id: str
        :param application_id: The application that owns the documents
        :type application_id: str
        :param filename: The file name of the documents to retrieve
        :type filename: str
        :param extension: The extension of the documents to retrieve
        :type extension: str
        :param mime_type: Filters by MIME type
        :type mime_type: str
        :param original_id: Filters by original id
        :type original_id: str
        :param tags: If specified, filters the documents by tag
        :type tags: List[str]
        :param created_after: If specified, keeps only documents created after given UTC timestamp (ISO 8601 format : yyyy-MM-ddThh:mm:ss)
        :type created_after: datetime
        :param created_before: If specified, keeps only documents created before given UTC timestamp (ISO 8601 format : yyyy-MM-ddThh:mm:ss)
        :type created_before: datetime
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(str, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'token',
            'document_id',
            'application_id',
            'filename',
            'extension',
            'mime_type',
            'original_id',
            'tags',
            'created_after',
            'created_before'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method admin_get_documents_zip" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('document_id') is not None:  # noqa: E501
            _query_params.append(('document_id', _params['document_id']))
        if _params.get('application_id') is not None:  # noqa: E501
            _query_params.append(('application_id', _params['application_id']))
        if _params.get('filename') is not None:  # noqa: E501
            _query_params.append(('filename', _params['filename']))
        if _params.get('extension') is not None:  # noqa: E501
            _query_params.append(('extension', _params['extension']))
        if _params.get('mime_type') is not None:  # noqa: E501
            _query_params.append(('mime_type', _params['mime_type']))
        if _params.get('original_id') is not None:  # noqa: E501
            _query_params.append(('original_id', _params['original_id']))
        if _params.get('tags') is not None:  # noqa: E501
            _query_params.append(('tags', _params['tags']))
            _collection_formats['tags'] = 'multi'
        if _params.get('created_after') is not None:  # noqa: E501
            _query_params.append(('created_after', _params['created_after']))
        if _params.get('created_before') is not None:  # noqa: E501
            _query_params.append(('created_before', _params['created_before']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['token']:
            _header_params['token'] = _params['token']

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/zip'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "str",
            '401': None,
            '403': None,
            '404': None,
        }

        return self.api_client.call_api(
            '/admin/document/zip', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def admin_tag_document(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[StrictStr, Field(..., description="The application to which the document to tag belongs")], document_id : Annotated[StrictStr, Field(..., description="The id of the document")], tag : Annotated[StrictStr, Field(..., description="The tag to add to the document")], **kwargs) -> Document:  # noqa: E501
        """Tags a document (admin only)  # noqa: E501

        Tags a document. This method is only accessible to admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_tag_document(token, application_id, document_id, tag, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: The application to which the document to tag belongs (required)
        :type application_id: str
        :param document_id: The id of the document (required)
        :type document_id: str
        :param tag: The tag to add to the document (required)
        :type tag: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Document
        """
        kwargs['_return_http_data_only'] = True
        return self.admin_tag_document_with_http_info(token, application_id, document_id, tag, **kwargs)  # noqa: E501

    @validate_arguments
    def admin_tag_document_with_http_info(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[StrictStr, Field(..., description="The application to which the document to tag belongs")], document_id : Annotated[StrictStr, Field(..., description="The id of the document")], tag : Annotated[StrictStr, Field(..., description="The tag to add to the document")], **kwargs):  # noqa: E501
        """Tags a document (admin only)  # noqa: E501

        Tags a document. This method is only accessible to admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_tag_document_with_http_info(token, application_id, document_id, tag, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: The application to which the document to tag belongs (required)
        :type application_id: str
        :param document_id: The id of the document (required)
        :type document_id: str
        :param tag: The tag to add to the document (required)
        :type tag: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Document, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'token',
            'application_id',
            'document_id',
            'tag'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method admin_tag_document" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['application_id']:
            _path_params['application_id'] = _params['application_id']
        if _params['document_id']:
            _path_params['document_id'] = _params['document_id']
        if _params['tag']:
            _path_params['tag'] = _params['tag']

        # process the query parameters
        _query_params = []

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['token']:
            _header_params['token'] = _params['token']

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "Document",
            '401': None,
            '404': None,
        }

        return self.api_client.call_api(
            '/admin/{application_id}/document/{document_id}/tag/{tag}', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def admin_transform_document_async(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[StrictStr, Field(..., description="The application id whose documents to transform belong to")], document_ids : Annotated[List[StrictStr], Field(..., description="Comma separated list of document ids to process")], output_type : Annotated[TransformTypes, Field(..., description="The output type. May be:   * image (extract one image for each page in a PDF file)   * text (Use OCR on an image to get convert it to text)     * To use an OCR different from the normal LEIA OCR, add \"ocr\":google_vision or \"ocr\":paddle in transform_params   * text_tree (text in the form of a JSON tree with information about text blocks and their position in the document)     * To use an OCR different from the normal LEIA OCR, add \"ocr\":google_vision or \"ocr\":paddle in transform_params   * autorotate (Rotates an image that contains text so that it is in readable orientation)   * trim (Trims white space around a document)   * merge (Merge multiple text documents into a single one with a carriage return '\\n' between them)   * split (Splits a text document into chunks of 1000 words and replaces all spacing characters by single spaces (this number can be changed by setting split_size to another value in transform_params)) ")], input_tag : Annotated[Optional[StrictStr], Field(description="The tag of the documents to process. If tag is present, document_ids should contain a single value, and the documents processed will be those where original_id=document_ids[0] and that contain the specified tag")] = None, output_tag : Annotated[Optional[StrictStr], Field(description="The tag to add to the documents resulting from the transformation")] = None, execute_after_id : Annotated[Optional[StrictStr], Field(description="The id of a job that must be in PROCESSED status before this one can be started (used to chain jobs even before the first ones are terminated). If the referenced job becomes FAILED or is CANCELED, this one will fail")] = None, page_range : Annotated[Optional[StrictStr], Field(description="The pages that should be used in previous job to process this one. Can only be used if execute_after_id is not null. Pages are indexed from 0. Syntax is the same as Python slices syntax (https://docs.python.org/3/whatsnew/2.3.html#extended-slices). Examples :   * Single positive integer : keep only this page (example 4 will keep only page 5 (Remember, pages are indexed from 0))   * Single negative integer : keep only this page, but starting from the end (example -4 will keep only page 7 if there are 10 total pages)   * Range (x:y) : keep only this range of pages (Including x but excluding y, indexed from 0)     Examples       * 2: will keep all pages starting from page 3       * :5 will keep only pages 1 to 5       * 2:5 will keep only pages 3, 4 and 5       * -4: will keep only pages 7 to 10 if there are 10 total pages)       * :-2 will keep only pages 1 to 8 if there are 10 total pages)       * -4:-2 will keep only pages 7 and 8 if there are 10 total pages)   * Stride (::w) : Keep 1 page every w pages starting at the first one (example ::2 will keep only odd pages)   * Range and stride (x:y:w) : Keep 1 page every w pages within range (x:y) (example 1::2 will keep only even pages) You can use multiple ranges of page at once, comma separated (For example, 0,2:5,-2:-1 keeps the 1st page, plus pages 3->5, plus the second to last page) ")] = None, callback_url : Annotated[Optional[StrictStr], Field(description="Callback URL that should be called when the job becomes PROCESSED/FAILED/CANCELED. This URL will be called with a HTTP POST method, and the Job object as the payload. Callback server must answer with either a 200 or 204 HTTP response, to acknowledge the callback. Any other response code will be considered as a failure to call the callback.")] = None, transform_params : Annotated[Optional[Dict[str, StrictStr]], Field(description="Free form parameters for the transformation")] = None, block_processing : Annotated[Optional[StrictBool], Field(description="If true, blocks processing on the job until /job/{id}/start is called. Default is false")] = None, transform_body : Annotated[Optional[TransformBody], Field(description="All the previous query parameters can also be passed as JSON in the body of the request")] = None, **kwargs) -> Job:  # noqa: E501
        """Asynchronously converts a document within Leia API (admin only)  # noqa: E501

        Asynchronously transforms a document from its current type to the output_type. May generate multiple new documents (for example converting a PDF to image will generate one new image document for each page of the PDF). Returns a Job, that will have to be polled to get the result. This method is only accessible to admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_transform_document_async(token, application_id, document_ids, output_type, input_tag, output_tag, execute_after_id, page_range, callback_url, transform_params, block_processing, transform_body, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: The application id whose documents to transform belong to (required)
        :type application_id: str
        :param document_ids: Comma separated list of document ids to process (required)
        :type document_ids: List[str]
        :param output_type: The output type. May be:   * image (extract one image for each page in a PDF file)   * text (Use OCR on an image to get convert it to text)     * To use an OCR different from the normal LEIA OCR, add \"ocr\":google_vision or \"ocr\":paddle in transform_params   * text_tree (text in the form of a JSON tree with information about text blocks and their position in the document)     * To use an OCR different from the normal LEIA OCR, add \"ocr\":google_vision or \"ocr\":paddle in transform_params   * autorotate (Rotates an image that contains text so that it is in readable orientation)   * trim (Trims white space around a document)   * merge (Merge multiple text documents into a single one with a carriage return '\\n' between them)   * split (Splits a text document into chunks of 1000 words and replaces all spacing characters by single spaces (this number can be changed by setting split_size to another value in transform_params))  (required)
        :type output_type: TransformTypes
        :param input_tag: The tag of the documents to process. If tag is present, document_ids should contain a single value, and the documents processed will be those where original_id=document_ids[0] and that contain the specified tag
        :type input_tag: str
        :param output_tag: The tag to add to the documents resulting from the transformation
        :type output_tag: str
        :param execute_after_id: The id of a job that must be in PROCESSED status before this one can be started (used to chain jobs even before the first ones are terminated). If the referenced job becomes FAILED or is CANCELED, this one will fail
        :type execute_after_id: str
        :param page_range: The pages that should be used in previous job to process this one. Can only be used if execute_after_id is not null. Pages are indexed from 0. Syntax is the same as Python slices syntax (https://docs.python.org/3/whatsnew/2.3.html#extended-slices). Examples :   * Single positive integer : keep only this page (example 4 will keep only page 5 (Remember, pages are indexed from 0))   * Single negative integer : keep only this page, but starting from the end (example -4 will keep only page 7 if there are 10 total pages)   * Range (x:y) : keep only this range of pages (Including x but excluding y, indexed from 0)     Examples       * 2: will keep all pages starting from page 3       * :5 will keep only pages 1 to 5       * 2:5 will keep only pages 3, 4 and 5       * -4: will keep only pages 7 to 10 if there are 10 total pages)       * :-2 will keep only pages 1 to 8 if there are 10 total pages)       * -4:-2 will keep only pages 7 and 8 if there are 10 total pages)   * Stride (::w) : Keep 1 page every w pages starting at the first one (example ::2 will keep only odd pages)   * Range and stride (x:y:w) : Keep 1 page every w pages within range (x:y) (example 1::2 will keep only even pages) You can use multiple ranges of page at once, comma separated (For example, 0,2:5,-2:-1 keeps the 1st page, plus pages 3->5, plus the second to last page) 
        :type page_range: str
        :param callback_url: Callback URL that should be called when the job becomes PROCESSED/FAILED/CANCELED. This URL will be called with a HTTP POST method, and the Job object as the payload. Callback server must answer with either a 200 or 204 HTTP response, to acknowledge the callback. Any other response code will be considered as a failure to call the callback.
        :type callback_url: str
        :param transform_params: Free form parameters for the transformation
        :type transform_params: object
        :param block_processing: If true, blocks processing on the job until /job/{id}/start is called. Default is false
        :type block_processing: bool
        :param transform_body: All the previous query parameters can also be passed as JSON in the body of the request
        :type transform_body: TransformBody
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Job
        """
        kwargs['_return_http_data_only'] = True
        return self.admin_transform_document_async_with_http_info(token, application_id, document_ids, output_type, input_tag, output_tag, execute_after_id, page_range, callback_url, transform_params, block_processing, transform_body, **kwargs)  # noqa: E501

    @validate_arguments
    def admin_transform_document_async_with_http_info(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[StrictStr, Field(..., description="The application id whose documents to transform belong to")], document_ids : Annotated[List[StrictStr], Field(..., description="Comma separated list of document ids to process")], output_type : Annotated[TransformTypes, Field(..., description="The output type. May be:   * image (extract one image for each page in a PDF file)   * text (Use OCR on an image to get convert it to text)     * To use an OCR different from the normal LEIA OCR, add \"ocr\":google_vision or \"ocr\":paddle in transform_params   * text_tree (text in the form of a JSON tree with information about text blocks and their position in the document)     * To use an OCR different from the normal LEIA OCR, add \"ocr\":google_vision or \"ocr\":paddle in transform_params   * autorotate (Rotates an image that contains text so that it is in readable orientation)   * trim (Trims white space around a document)   * merge (Merge multiple text documents into a single one with a carriage return '\\n' between them)   * split (Splits a text document into chunks of 1000 words and replaces all spacing characters by single spaces (this number can be changed by setting split_size to another value in transform_params)) ")], input_tag : Annotated[Optional[StrictStr], Field(description="The tag of the documents to process. If tag is present, document_ids should contain a single value, and the documents processed will be those where original_id=document_ids[0] and that contain the specified tag")] = None, output_tag : Annotated[Optional[StrictStr], Field(description="The tag to add to the documents resulting from the transformation")] = None, execute_after_id : Annotated[Optional[StrictStr], Field(description="The id of a job that must be in PROCESSED status before this one can be started (used to chain jobs even before the first ones are terminated). If the referenced job becomes FAILED or is CANCELED, this one will fail")] = None, page_range : Annotated[Optional[StrictStr], Field(description="The pages that should be used in previous job to process this one. Can only be used if execute_after_id is not null. Pages are indexed from 0. Syntax is the same as Python slices syntax (https://docs.python.org/3/whatsnew/2.3.html#extended-slices). Examples :   * Single positive integer : keep only this page (example 4 will keep only page 5 (Remember, pages are indexed from 0))   * Single negative integer : keep only this page, but starting from the end (example -4 will keep only page 7 if there are 10 total pages)   * Range (x:y) : keep only this range of pages (Including x but excluding y, indexed from 0)     Examples       * 2: will keep all pages starting from page 3       * :5 will keep only pages 1 to 5       * 2:5 will keep only pages 3, 4 and 5       * -4: will keep only pages 7 to 10 if there are 10 total pages)       * :-2 will keep only pages 1 to 8 if there are 10 total pages)       * -4:-2 will keep only pages 7 and 8 if there are 10 total pages)   * Stride (::w) : Keep 1 page every w pages starting at the first one (example ::2 will keep only odd pages)   * Range and stride (x:y:w) : Keep 1 page every w pages within range (x:y) (example 1::2 will keep only even pages) You can use multiple ranges of page at once, comma separated (For example, 0,2:5,-2:-1 keeps the 1st page, plus pages 3->5, plus the second to last page) ")] = None, callback_url : Annotated[Optional[StrictStr], Field(description="Callback URL that should be called when the job becomes PROCESSED/FAILED/CANCELED. This URL will be called with a HTTP POST method, and the Job object as the payload. Callback server must answer with either a 200 or 204 HTTP response, to acknowledge the callback. Any other response code will be considered as a failure to call the callback.")] = None, transform_params : Annotated[Optional[Dict[str, StrictStr]], Field(description="Free form parameters for the transformation")] = None, block_processing : Annotated[Optional[StrictBool], Field(description="If true, blocks processing on the job until /job/{id}/start is called. Default is false")] = None, transform_body : Annotated[Optional[TransformBody], Field(description="All the previous query parameters can also be passed as JSON in the body of the request")] = None, **kwargs):  # noqa: E501
        """Asynchronously converts a document within Leia API (admin only)  # noqa: E501

        Asynchronously transforms a document from its current type to the output_type. May generate multiple new documents (for example converting a PDF to image will generate one new image document for each page of the PDF). Returns a Job, that will have to be polled to get the result. This method is only accessible to admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_transform_document_async_with_http_info(token, application_id, document_ids, output_type, input_tag, output_tag, execute_after_id, page_range, callback_url, transform_params, block_processing, transform_body, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: The application id whose documents to transform belong to (required)
        :type application_id: str
        :param document_ids: Comma separated list of document ids to process (required)
        :type document_ids: List[str]
        :param output_type: The output type. May be:   * image (extract one image for each page in a PDF file)   * text (Use OCR on an image to get convert it to text)     * To use an OCR different from the normal LEIA OCR, add \"ocr\":google_vision or \"ocr\":paddle in transform_params   * text_tree (text in the form of a JSON tree with information about text blocks and their position in the document)     * To use an OCR different from the normal LEIA OCR, add \"ocr\":google_vision or \"ocr\":paddle in transform_params   * autorotate (Rotates an image that contains text so that it is in readable orientation)   * trim (Trims white space around a document)   * merge (Merge multiple text documents into a single one with a carriage return '\\n' between them)   * split (Splits a text document into chunks of 1000 words and replaces all spacing characters by single spaces (this number can be changed by setting split_size to another value in transform_params))  (required)
        :type output_type: TransformTypes
        :param input_tag: The tag of the documents to process. If tag is present, document_ids should contain a single value, and the documents processed will be those where original_id=document_ids[0] and that contain the specified tag
        :type input_tag: str
        :param output_tag: The tag to add to the documents resulting from the transformation
        :type output_tag: str
        :param execute_after_id: The id of a job that must be in PROCESSED status before this one can be started (used to chain jobs even before the first ones are terminated). If the referenced job becomes FAILED or is CANCELED, this one will fail
        :type execute_after_id: str
        :param page_range: The pages that should be used in previous job to process this one. Can only be used if execute_after_id is not null. Pages are indexed from 0. Syntax is the same as Python slices syntax (https://docs.python.org/3/whatsnew/2.3.html#extended-slices). Examples :   * Single positive integer : keep only this page (example 4 will keep only page 5 (Remember, pages are indexed from 0))   * Single negative integer : keep only this page, but starting from the end (example -4 will keep only page 7 if there are 10 total pages)   * Range (x:y) : keep only this range of pages (Including x but excluding y, indexed from 0)     Examples       * 2: will keep all pages starting from page 3       * :5 will keep only pages 1 to 5       * 2:5 will keep only pages 3, 4 and 5       * -4: will keep only pages 7 to 10 if there are 10 total pages)       * :-2 will keep only pages 1 to 8 if there are 10 total pages)       * -4:-2 will keep only pages 7 and 8 if there are 10 total pages)   * Stride (::w) : Keep 1 page every w pages starting at the first one (example ::2 will keep only odd pages)   * Range and stride (x:y:w) : Keep 1 page every w pages within range (x:y) (example 1::2 will keep only even pages) You can use multiple ranges of page at once, comma separated (For example, 0,2:5,-2:-1 keeps the 1st page, plus pages 3->5, plus the second to last page) 
        :type page_range: str
        :param callback_url: Callback URL that should be called when the job becomes PROCESSED/FAILED/CANCELED. This URL will be called with a HTTP POST method, and the Job object as the payload. Callback server must answer with either a 200 or 204 HTTP response, to acknowledge the callback. Any other response code will be considered as a failure to call the callback.
        :type callback_url: str
        :param transform_params: Free form parameters for the transformation
        :type transform_params: object
        :param block_processing: If true, blocks processing on the job until /job/{id}/start is called. Default is false
        :type block_processing: bool
        :param transform_body: All the previous query parameters can also be passed as JSON in the body of the request
        :type transform_body: TransformBody
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Job, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'token',
            'application_id',
            'document_ids',
            'output_type',
            'input_tag',
            'output_tag',
            'execute_after_id',
            'page_range',
            'callback_url',
            'transform_params',
            'block_processing',
            'transform_body'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method admin_transform_document_async" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['application_id']:
            _path_params['application_id'] = _params['application_id']
        if _params['document_ids']:
            _path_params['document_ids'] = _params['document_ids']
            _collection_formats['document_ids'] = 'csv'
        if _params['output_type']:
            _path_params['output_type'] = _params['output_type']

        # process the query parameters
        _query_params = []
        if _params.get('input_tag') is not None:  # noqa: E501
            _query_params.append(('input_tag', _params['input_tag']))
        if _params.get('output_tag') is not None:  # noqa: E501
            _query_params.append(('output_tag', _params['output_tag']))
        if _params.get('execute_after_id') is not None:  # noqa: E501
            _query_params.append(('execute_after_id', _params['execute_after_id']))
        if _params.get('page_range') is not None:  # noqa: E501
            _query_params.append(('page_range', _params['page_range']))
        if _params.get('callback_url') is not None:  # noqa: E501
            _query_params.append(('callback_url', _params['callback_url']))
        if _params.get('transform_params') is not None:  # noqa: E501
            _query_params.append(('transform_params', _params['transform_params']))
        if _params.get('block_processing') is not None:  # noqa: E501
            _query_params.append(('block_processing', _params['block_processing']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['token']:
            _header_params['token'] = _params['token']

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None
        if _params['transform_body']:
            _body_params = _params['transform_body']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "Job",
            '401': None,
            '403': None,
            '404': None,
        }

        return self.api_client.call_api(
            '/admin/{application_id}/document/{document_ids}/transform/{output_type}', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def admin_untag_document(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[StrictStr, Field(..., description="The application to which the document to untag belongs")], document_id : Annotated[StrictStr, Field(..., description="The id of the document")], tag : Annotated[StrictStr, Field(..., description="The tag to delete from the document")], **kwargs) -> None:  # noqa: E501
        """Untags an document (admin only)  # noqa: E501

        Untags a document. This method is only accessible to admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_untag_document(token, application_id, document_id, tag, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: The application to which the document to untag belongs (required)
        :type application_id: str
        :param document_id: The id of the document (required)
        :type document_id: str
        :param tag: The tag to delete from the document (required)
        :type tag: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs['_return_http_data_only'] = True
        return self.admin_untag_document_with_http_info(token, application_id, document_id, tag, **kwargs)  # noqa: E501

    @validate_arguments
    def admin_untag_document_with_http_info(self, token : Annotated[StrictStr, Field(..., description="The login token obtained via POST /login/")], application_id : Annotated[StrictStr, Field(..., description="The application to which the document to untag belongs")], document_id : Annotated[StrictStr, Field(..., description="The id of the document")], tag : Annotated[StrictStr, Field(..., description="The tag to delete from the document")], **kwargs):  # noqa: E501
        """Untags an document (admin only)  # noqa: E501

        Untags a document. This method is only accessible to admins  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.admin_untag_document_with_http_info(token, application_id, document_id, tag, async_req=True)
        >>> result = thread.get()

        :param token: The login token obtained via POST /login/ (required)
        :type token: str
        :param application_id: The application to which the document to untag belongs (required)
        :type application_id: str
        :param document_id: The id of the document (required)
        :type document_id: str
        :param tag: The tag to delete from the document (required)
        :type tag: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = [
            'token',
            'application_id',
            'document_id',
            'tag'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method admin_untag_document" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['application_id']:
            _path_params['application_id'] = _params['application_id']
        if _params['document_id']:
            _path_params['document_id'] = _params['document_id']
        if _params['tag']:
            _path_params['tag'] = _params['tag']

        # process the query parameters
        _query_params = []

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['token']:
            _header_params['token'] = _params['token']

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            '/admin/{application_id}/document/{document_id}/tag/{tag}', 'DELETE',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    LEIA RESTful API for AI

    Leia API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Contact: contact@leia.io
    Generated by: https://openapi-generator.tech
"""


from inspect import getfullargspec
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class ResultTypes(str, Enum):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    allowed enum values
    """

    DOCUMENT = 'document'
    LIST_DOCUMENT = 'list[document]'
    OBJECT = 'object'
    CLASSIFICATION = 'classification'
    DICT_OBJECT = 'dict[object]'
    DICT_CLASSIFICATION = 'dict[classification]'
    MODEL = 'model'
    CONDITIONAL = 'conditional'




# coding: utf-8

"""
    LEIA RESTful API for AI

    Leia API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Contact: contact@leia.io
    Generated by: https://openapi-generator.tech
"""


from __future__ import annotations
from inspect import getfullargspec
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Any, Dict, Optional
from pydantic import BaseModel, StrictBool, StrictInt, StrictStr
from leiaapi.generated.models.application_types import ApplicationTypes

class Application(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """
    api_key: Optional[StrictStr] = None
    application_name: Optional[StrictStr] = None
    application_type: ApplicationTypes = ...
    creation_time: Optional[datetime] = None
    dedicated_workers: Optional[StrictBool] = None
    dedicated_workers_max_models: Optional[StrictInt] = None
    dedicated_workers_ttl: Optional[StrictInt] = None
    default_job_callback_url: Optional[StrictStr] = None
    email: StrictStr = ...
    first_name: StrictStr = ...
    id: Optional[StrictStr] = None
    job_counts: Optional[Dict[str, Any]] = None
    last_name: StrictStr = ...
    __properties = ["api_key", "application_name", "application_type", "creation_time", "dedicated_workers", "dedicated_workers_max_models", "dedicated_workers_ttl", "default_job_callback_url", "email", "first_name", "id", "job_counts", "last_name"]

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Application:
        """Create an instance of Application from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Application:
        """Create an instance of Application from a dict"""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Application.parse_obj(obj)

        _obj = Application.parse_obj({
            "api_key": obj.get("api_key"),
            "application_name": obj.get("application_name"),
            "application_type": obj.get("application_type"),
            "creation_time": obj.get("creation_time"),
            "dedicated_workers": obj.get("dedicated_workers"),
            "dedicated_workers_max_models": obj.get("dedicated_workers_max_models"),
            "dedicated_workers_ttl": obj.get("dedicated_workers_ttl"),
            "default_job_callback_url": obj.get("default_job_callback_url"),
            "email": obj.get("email"),
            "first_name": obj.get("first_name"),
            "id": obj.get("id"),
            "job_counts": obj.get("job_counts"),
            "last_name": obj.get("last_name")
        })
        return _obj




# coding: utf-8

# flake8: noqa

"""
    LEIA RESTful API for AI

    Leia API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Contact: contact@leia.io
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from leiaapi.generated.api.annotation_api import AnnotationApi
from leiaapi.generated.api.application_api import ApplicationApi
from leiaapi.generated.api.application_admin_api import ApplicationAdminApi
from leiaapi.generated.api.document_api import DocumentApi
from leiaapi.generated.api.document_admin_api import DocumentAdminApi
from leiaapi.generated.api.health_api import HealthApi
from leiaapi.generated.api.job_api import JobApi
from leiaapi.generated.api.job_admin_api import JobAdminApi
from leiaapi.generated.api.model_api import ModelApi
from leiaapi.generated.api.model_admin_api import ModelAdminApi
from leiaapi.generated.api.worker_api import WorkerApi
from leiaapi.generated.api.worker_admin_api import WorkerAdminApi

# import ApiClient
from leiaapi.generated.api_client import ApiClient
from leiaapi.generated.configuration import Configuration
from leiaapi.generated.exceptions import OpenApiException
from leiaapi.generated.exceptions import ApiTypeError
from leiaapi.generated.exceptions import ApiValueError
from leiaapi.generated.exceptions import ApiKeyError
from leiaapi.generated.exceptions import ApiAttributeError
from leiaapi.generated.exceptions import ApiException
# import models into sdk package
from leiaapi.generated.models.annotation import Annotation
from leiaapi.generated.models.annotation_types import AnnotationTypes
from leiaapi.generated.models.application import Application
from leiaapi.generated.models.application_types import ApplicationTypes
from leiaapi.generated.models.apply_body import ApplyBody
from leiaapi.generated.models.classification import Classification
from leiaapi.generated.models.conditional_body import ConditionalBody
from leiaapi.generated.models.document import Document
from leiaapi.generated.models.format_types import FormatTypes
from leiaapi.generated.models.job import Job
from leiaapi.generated.models.job_result import JobResult
from leiaapi.generated.models.job_types import JobTypes
from leiaapi.generated.models.login_body import LoginBody
from leiaapi.generated.models.login_token import LoginToken
from leiaapi.generated.models.model import Model
from leiaapi.generated.models.model_input_types import ModelInputTypes
from leiaapi.generated.models.model_types import ModelTypes
from leiaapi.generated.models.result_types import ResultTypes
from leiaapi.generated.models.speed import Speed
from leiaapi.generated.models.statuses import Statuses
from leiaapi.generated.models.train_body import TrainBody
from leiaapi.generated.models.transform_body import TransformBody
from leiaapi.generated.models.transform_types import TransformTypes
from leiaapi.generated.models.worker import Worker



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from leiaapi.generated.api.annotation_api import AnnotationApi
from leiaapi.generated.api.application_api import ApplicationApi
from leiaapi.generated.api.application_admin_api import ApplicationAdminApi
from leiaapi.generated.api.document_api import DocumentApi
from leiaapi.generated.api.document_admin_api import DocumentAdminApi
from leiaapi.generated.api.health_api import HealthApi
from leiaapi.generated.api.job_api import JobApi
from leiaapi.generated.api.job_admin_api import JobAdminApi
from leiaapi.generated.api.model_api import ModelApi
from leiaapi.generated.api.model_admin_api import ModelAdminApi
from leiaapi.generated.api.worker_api import WorkerApi
from leiaapi.generated.api.worker_admin_api import WorkerAdminApi



# coding: utf-8

"""
    LEIA RESTful API for AI

    Leia API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Contact: contact@leia.io
    Generated by: https://openapi-generator.tech
"""

from __future__ import absolute_import

import os
import unittest
import time

from leiaapi.generated import Statuses, TransformTypes, ApiClient, Configuration

import leiaapi.generated
from leiaapi.generated.api.application_api import ApplicationApi  # noqa: E501
from leiaapi.generated.exceptions import NotFoundException
from leiaapi.generated.models import LoginBody


class TestApplicationApi(unittest.TestCase):
    """ApplicationApi unit test stubs"""

    def setUp(self):
        self.client = ApiClient(Configuration(host="https://dev.leia.id360docaposte.com/leia/1.0.0"))
        self.api = leiaapi.generated.api.ApplicationApi(self.client)  # noqa: E501
        self.model_api = leiaapi.generated.api.ModelApi(self.client)  # noqa: E501
        self.model_admin_api = leiaapi.generated.api.ModelAdminApi(self.client)  # noqa: E501
        self.document_api = leiaapi.generated.api.DocumentApi(self.client)
        self.job_api = leiaapi.generated.api.JobApi(self.client)
        self.application_admin_api = leiaapi.generated.api.ApplicationAdminApi(self.client)

    def tearDown(self):
        pass

    def test_login_application_post(self):
        """Test case for login_application_post

        Logs into Leia API  # noqa: E501
        """
        print(f"The test will be done against the dev environment({self.client.configuration._base_path})")
        api_key = os.environ.get("LEIA_API_KEY", None)
        self.assertIsNotNone(api_key, msg="Please set the environment variable LEIA_API_KEY to your api key")
        login_token = self.api.login_application_post(LoginBody(api_key=api_key))
        token = login_token.token
        self.assertIsNotNone(token)
        app = self.api.who_am_i(token=str(token))
        self.assertIsNotNone(app.application_type)
        self.assertIsNotNone(app.creation_time)
        print("Login OK")


        new_doc = self.document_api.create_document(token, "test-sdk.txt", "test sdk 123".encode("UTF8"), ttl=600, tags=["test-sdk"])
        self.assertIsNotNone(new_doc.id)
        print("Create document OK")

        docs = self.document_api.get_documents(token)
        self.assertGreater(len(docs), 0)
        print("Get documents 1 OK")

        docs = self.document_api.get_documents(token, document_id=new_doc.id)
        self.assertEqual(len(docs), 1)
        print("Get documents 2 OK")

        docs_2 = self.document_api.get_documents(token,
                                                 document_id=new_doc.id,
                                                 filename=new_doc.filename,
                                                 extension=new_doc.extension,
                                                 mime_type=new_doc.mime_type,
                                                 original_id=new_doc.original_id,
                                                 tags=new_doc.tags,
                                                 # This does not work because the datetime serailizer is not compatible
                                                 # created_before=docs[0].creation_time + relativedelta(hours=1),
                                                 # created_after=docs[0].creation_time - relativedelta(hours=1),
                                                 sort="filename",
                                                 offset=0,
                                                 limit=1)
        self.assertGreater(len(docs), 0)
        self.assertEqual(docs_2[0], docs[0])
        print("Get documents 3 OK")

        doc = self.document_api.get_document(token, docs[0].id)
        self.assertEqual(doc, docs[0])
        print("Get document OK")

        docs_zip = self.document_api.get_documents_zip(token,
                                                       document_id=new_doc.id,
                                                       filename=new_doc.filename,
                                                       extension=new_doc.extension,
                                                       mime_type=new_doc.mime_type,
                                                       original_id=new_doc.original_id,
                                                       tags=new_doc.tags,
                                                       # created_before=docs[0].creation_time + relativedelta(hours=1),
                                                       # created_after=docs[0].creation_time - relativedelta(hours=1)
                                                       )
        self.assertGreater(len(docs_zip), 0)
        self.assertEqual(docs_zip[0], 0x50)
        self.assertEqual(docs_zip[1], 0x4B)
        print("Get documents ZIP OK")


        data = self.document_api.get_document_contents(token, new_doc.id)
        self.assertEqual(data, "test sdk 123".encode("UTF8"))
        print("Get document content OK")

        apps = self.application_admin_api.admin_get_applications(token)
        self.assertGreater(len(apps), 0)
        print("Get applications OK")

        models = self.model_api.get_models(token)
        model = min(models, key= lambda x: x.size)
        self.assertGreater(len(models), 0)
        print("Get models OK")

        model_data = self.model_admin_api.admin_get_model_contents(token, model.application_id, model.id)
        self.assertEqual(model_data[0], 0x50)
        self.assertEqual(model_data[1], 0x4B)
        print("Get model content OK")

        job = self.model_api.apply_model_async(token, model.id, [new_doc.id], block_processing=True)
        print("Apply model OK")
        self.job_api.start_job(token, job.id)
        while job.status not in [Statuses.FAILED, Statuses.PROCESSED]:
            job = self.job_api.get_job(token, job.id)
            time.sleep(1)
        self.assertIn(job.status, [Statuses.FAILED, Statuses.PROCESSED])
        print("Apply model start OK")

        job = self.document_api.transform_document_async(token, [new_doc.id, new_doc.id], TransformTypes.MERGE, block_processing=True)
        print("Transform doc MERGE OK")
        self.job_api.start_job(token, job.id)
        while job.status not in [Statuses.FAILED, Statuses.PROCESSED]:
            job = self.job_api.get_job(token, job.id)
            time.sleep(1)
        self.assertIn(job.status, [Statuses.FAILED, Statuses.PROCESSED])
        print("Transform doc MERGE start OK")

        docs_pdf = self.document_api.get_documents(token,
                                                 mime_type="application/pdf",
                                                 limit=1)
        self.assertGreater(len(docs_pdf), 0)
        print("Get document pdf OK")


        job = self.document_api.transform_document_async(token, [docs_pdf[0].id], TransformTypes.IMAGE, block_processing=True)
        print("Transform doc IMAGE start OK")
        self.job_api.start_job(token, job.id)
        while job.status not in [Statuses.FAILED, Statuses.PROCESSED]:
            job = self.job_api.get_job(token, job.id)
            time.sleep(1)
        self.assertIn(job.status, [Statuses.FAILED, Statuses.PROCESSED])
        print("Transform doc IMAGE OK")


        job = self.document_api.transform_document_async(token, job.document_ids, TransformTypes.TEXT, block_processing=True, execute_after_id=job.id)
        print("Transform doc TEXT start OK")
        self.job_api.start_job(token, job.id)
        while job.status not in [Statuses.FAILED, Statuses.PROCESSED]:
            job = self.job_api.get_job(token, job.id)
            time.sleep(1)
        self.assertIn(job.status, [Statuses.FAILED, Statuses.PROCESSED])
        print("Transform doc TEXT OK")

        self.document_api.delete_document(token, new_doc.id)
        print("Delete doc OK")
        self.assertRaises(NotFoundException, self.document_api.get_document, token, document_id=new_doc.id)
        print("Get deleted doc OK")

        self.api.logout_application(token=str(token))
        print("logout OK")


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    LEIA RESTful API for AI

    Leia API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Contact: contact@leia.io
    Generated by: https://openapi-generator.tech
"""


from __future__ import annotations
from inspect import getfullargspec
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic import BaseModel, StrictBool, StrictStr

class TransformBody(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """
    block_processing: Optional[StrictBool] = None
    callback_headers: Optional[Dict[str, Any]] = None
    callback_url: Optional[StrictStr] = None
    execute_after_id: Optional[StrictStr] = None
    input_tag: Optional[StrictStr] = None
    output_tag: Optional[StrictStr] = None
    page_range: Optional[StrictStr] = None
    transform_params: Optional[Dict[str, Any]] = None
    __properties = ["block_processing", "callback_headers", "callback_url", "execute_after_id", "input_tag", "output_tag", "page_range", "transform_params"]

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TransformBody:
        """Create an instance of TransformBody from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TransformBody:
        """Create an instance of TransformBody from a dict"""
        if obj is None:
            return None

        if type(obj) is not dict:
            return TransformBody.parse_obj(obj)

        _obj = TransformBody.parse_obj({
            "block_processing": obj.get("block_processing"),
            "callback_headers": obj.get("callback_headers"),
            "callback_url": obj.get("callback_url"),
            "execute_after_id": obj.get("execute_after_id"),
            "input_tag": obj.get("input_tag"),
            "output_tag": obj.get("output_tag"),
            "page_range": obj.get("page_range"),
            "transform_params": obj.get("transform_params")
        })
        return _obj




import logging
from typing import Optional

import leiaapi.generated.api_client
import leiaapi.generated.rest
from leiaapi.generated import ApiClient
from .scheduler import scheduled, Scheduler
from ..generated.api import ApplicationApi
from ..generated.models import Application, LoginBody, LoginToken

logger = logging.getLogger(__name__)

TIME_BETWEEN_TOKEN_UPDATE = 300

# this is the __call_api method
old__call_api = leiaapi.generated.api_client.ApiClient._ApiClient__call_api


def __call_api_with_auto_login(self, resource_path, method, path_params=None,
                  query_params=None, header_params=None, *args, **kwargs):
    try:
        return old__call_api(self, resource_path, method, path_params, query_params, header_params, *args, **kwargs)
    except leiaapi.generated.rest.ApiException as e:
        if resource_path == '/login':
            raise
        if e.status != 401 or 'token' not in header_params:
            # It's not a login error, we do not care for it
            # Or there is no token to send in the request, and we do not care either
            raise
        logger.info("Leia Token is invalid or non existent, trying to refresh it")
        SessionManager.DEFAULT_SESSION.login()

        header_params['token'] = SessionManager.DEFAULT_SESSION.token
        return old__call_api(self, resource_path, method, path_params, query_params, header_params, *args, **kwargs)

# We override the initial __call_api method to be able to automatically login if the token is not valid
leiaapi.generated.api_client.ApiClient._ApiClient__call_api = __call_api_with_auto_login


class SessionManager:
    DEFAULT_SESSION: 'SessionManager' = None

    def __init__(self, api_key: str, client: Optional[ApiClient] = ApiClient(), auto_update_token: bool = True):
        """
        Create a SessionManager to manage the session with Leia.io
        :param api_key: The API Key to connect to api.leia.io
        :param client: A ApiClient object configure with the information of the server
        :param auto_update_token: Set to False to not update the token validity automatically
        """
        super().__init__()
        self._api_key: str = api_key
        self._client: Optional[ApiClient] = client
        self._token: Optional[str] = None
        self._application_api: ApplicationApi = ApplicationApi(api_client=self.client)
        self._application: Optional[Application] = None
        self._scheduler: Optional[Scheduler] = None
        self._auto_update_token = auto_update_token

    @property
    def api_key(self):
        return self._api_key

    @api_key.setter
    def api_key(self, value):
        self._api_key = value

    @property
    def client(self):
        return self._client

    @client.setter
    def client(self, value):
        self._client = value
        self._application_api = ApplicationApi(api_client=self.client)

    @property
    def token(self):
        return self._token

    @property
    def application_api(self):
        return self._application_api

    @property
    def application(self):
        return self._application

    @property
    def scheduler(self):
        return self._scheduler

    def set_as_default(self):
        SessionManager.DEFAULT_SESSION = self
        return self

    def login(self):
        login = self._application_api.login_application_post(LoginBody(api_key=self.api_key))
        self._application: Optional[Application] = login.application
        self._token: Optional[str] = login.token

        if self._auto_update_token:
            @scheduled(TIME_BETWEEN_TOKEN_UPDATE)
            def renew():
                self._application_api.who_am_i(token=self.token)

            self._scheduler = renew
        return self

    def logout(self):
        if self.scheduler is not None:
            self.scheduler.cancel()
        self._application_api.logout_application(self.token)
        self._token = None
        return self


# coding: utf-8

"""
    LEIA RESTful API for AI

    Leia API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Contact: contact@leia.io
    Generated by: https://openapi-generator.tech
"""


from inspect import getfullargspec
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class Statuses(str, Enum):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    allowed enum values
    """

    BLOCKED = 'BLOCKED'
    WAITING = 'WAITING'
    READY = 'READY'
    STARTING = 'STARTING'
    LOADING_MODEL = 'LOADING_MODEL'
    PROCESSING = 'PROCESSING'
    PROCESSED = 'PROCESSED'
    FAILED = 'FAILED'
    CANCELED = 'CANCELED'

